@@ -12,12 +12,13 @@ from django.core.exceptions import ImproperlyConfigured
 
 logger = logging.getLogger('django.contrib.gis')
 
+# Custom library path set?
 if 'linux' in sys.platform:
     lib_path = os.path.join(sys.prefix, 'lib', 'libgdal.so')
 elif 'darwin' in sys.platform:
-    lib_path = os.path.join(sys.prefix, 'lib', 'libgdal.1.dylib')
+    lib_path = os.path.join(sys.prefix, 'lib', 'libgdal.dylib')
 elif os.name == 'nt':
-    lib_path = os.path.join(sys.prefix, 'Library', 'bin', 'gdal111.dll')
+    lib_path = os.path.join(sys.prefix, 'Library', 'bin', 'gdal201.dll')
 
 if lib_path:
     lib_names = None
