@@ -237,6 +237,9 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
         """
         return data.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return name not in data
+
     def id_for_label(self, id_):
         """
         Returns the HTML ID attribute of this Widget for use by a <label>,
@@ -249,6 +252,9 @@ class Widget(six.with_metaclass(RenameWidgetMethods)):
         """
         return id_
 
+    def use_required_attribute(self, initial):
+        return not self.is_hidden
+
 
 class Input(Widget):
     """
@@ -347,6 +353,9 @@ class FileInput(Input):
         "File widgets take data from FILES, not POST"
         return files.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return name not in files
+
 
 FILE_INPUT_CONTRADICTION = object()
 
@@ -428,6 +437,9 @@ class ClearableFileInput(FileInput):
             return False
         return upload
 
+    def use_required_attribute(self, initial):
+        return super(ClearableFileInput, self).use_required_attribute(initial) and not initial
+
 
 class Textarea(Widget):
     def __init__(self, attrs=None):
@@ -501,6 +513,11 @@ class CheckboxInput(Widget):
             value = values.get(value.lower(), value)
         return bool(value)
 
+    def value_omitted_from_data(self, data, files, name):
+        # HTML checkboxes don't appear in POST data if not checked, so it's
+        # never known if the value is actually omitted.
+        return False
+
 
 class Select(Widget):
     allow_multiple_selected = False
@@ -693,8 +710,11 @@ class ChoiceFieldRenderer(object):
         self.choices = choices
 
     def __getitem__(self, idx):
-        choice = list(self.choices)[idx]  # Let the IndexError propagate
-        return self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)
+        return list(self)[idx]
+
+    def __iter__(self):
+        for idx, choice in enumerate(self.choices):
+            yield self.choice_input_class(self.name, self.value, self.attrs.copy(), choice, idx)
 
     def __str__(self):
         return self.render()
@@ -770,7 +790,7 @@ class RendererMixin(object):
     def id_for_label(self, id_):
         # Widgets using this RendererMixin are made of a collection of
         # subwidgets, each with their own <label>, and distinct ID.
-        # The IDs are made distinct by y "_X" suffix, where X is the zero-based
+        # The IDs are made distinct by a "_X" suffix, where X is the zero-based
         # index of the choice field. Thus, the label for the main widget should
         # reference the first subwidget, hence the "_0" suffix.
         if id_:
@@ -787,12 +807,15 @@ class CheckboxSelectMultiple(RendererMixin, SelectMultiple):
     renderer = CheckboxFieldRenderer
     _empty_value = []
 
-    def build_attrs(self, extra_attrs=None, **kwargs):
-        attrs = super(CheckboxSelectMultiple, self).build_attrs(extra_attrs, **kwargs)
-        # Remove the 'required' attribute because browser validation would
+    def use_required_attribute(self, initial):
+        # Don't use the 'required' attribute because browser validation would
         # require all checkboxes to be checked instead of at least one.
-        attrs.pop('required', None)
-        return attrs
+        return False
+
+    def value_omitted_from_data(self, data, files, name):
+        # HTML checkboxes don't appear in POST data if not checked, so it's
+        # never known if the value is actually omitted.
+        return False
 
 
 class MultiWidget(Widget):
@@ -860,6 +883,12 @@ class MultiWidget(Widget):
     def value_from_datadict(self, data, files, name):
         return [widget.value_from_datadict(data, files, name + '_%s' % i) for i, widget in enumerate(self.widgets)]
 
+    def value_omitted_from_data(self, data, files, name):
+        return all(
+            widget.value_omitted_from_data(data, files, name + '_%s' % i)
+            for i, widget in enumerate(self.widgets)
+        )
+
     def format_output(self, rendered_widgets):
         """
         Given a list of rendered widgets (as strings), returns a Unicode string
@@ -1045,6 +1074,12 @@ class SelectDateWidget(Widget):
                 return '%s-%s-%s' % (y, m, d)
         return data.get(name)
 
+    def value_omitted_from_data(self, data, files, name):
+        return not any(
+            ('{}_{}'.format(name, interval) in data)
+            for interval in ('year', 'month', 'day')
+        )
+
     def create_select(self, name, field, value, val, choices, none_value):
         if 'id' in self.attrs:
             id_ = self.attrs['id']
